/**
 * Simple UI using 'bare' processing
 *
 * Display colorwheel and some simple menu buttons, 'slider'
 *
 * Revision 1.0, Updated: 2016-02-12 P. Peters
 */

final int border = 30;

// sizes and positions of GUI elements depend on screen dimensions
int wheelSize;
int menuStart;
int buttonHeight;
int textSize;

// variables influenced by GUI actions
int func = 1;
int speed = 1;
int bgcolor = 255;

void setup() {
  // Java processing needs size setting, Android does not...
   // size(300, 480);

  // make colorwheel size dependent on space available on screen
  wheelSize = width-2*border;
  // menu y-position depends on colorwheel size
  menuStart = wheelSize+2*border;
  buttonHeight = height/12;
  textSize = height/24;
  textSize(textSize);
}

void draw() {
  // set background to color picked on colorwheel
  background(bgcolor);
  // draw color wheel or bar
  drawColorSelector(false);
  // draw menu
  drawMenu();
  // display other GUI variable results
  text("Speed: "+nfp(speed, 2, 0)+" function: "+nfp(func, 2, 0), 10, menuStart+3*buttonHeight);
}

// draw color selector
void drawColorSelector(boolean circle) {
  // set HSB mode
  colorMode(HSB, 360, 100, 100); 
  stroke(255);
  if (circle) drawColorWheel();
  else drawColorBar();
}

// draw color wheel
void drawColorWheel() { 
  float theta;
  ellipseMode(CENTER);
  //draw 360 arcs around the center
  for (theta = -PI; theta < PI; theta += TWO_PI/180) {
    //map the color into the wheel using segments
    float hue = map(theta, -PI, PI, 0, 360);
    //fill in the colour 
    fill(hue, 100, 100);
    //create the arc
    arc(width/2, border+wheelSize/2, wheelSize, wheelSize, theta, theta + TWO_PI/180);
  }
}

void drawColorBar() {
  // draw color bar
  for (int i=0; i<width; i++) {  
    // set the colour dependent on x-position 
    stroke(map(i, 0, width, 0, 360), 100, 100);
    line(i, border, i, menuStart-border);
  }
}

void drawMenu() {
  //draw simple 'menu'
  stroke(0);
  // top line
  line(0, menuStart, width, menuStart);
  // end of 'slider' line = start of buttons line
  line(0, menuStart+buttonHeight, width, menuStart+buttonHeight);
  // divide width in 8 equal pieces delimited by lines
  for (int i=1; i<8; i++) {
    line(i*(width/8), menuStart+buttonHeight, i*(width/8), menuStart+2*buttonHeight);
  }
  // draw bottom of buttons line
  line(0, menuStart+2*buttonHeight, width, menuStart+2*buttonHeight);
}


void mouseReleased() {
  if (mouseY < menuStart) {
    int c = get(mouseX, mouseY);
    bgcolor = c;
  } else {
    if (mouseY < menuStart+buttonHeight) {
      speed = 1+mouseX/4;// minimum delay is 1
    } else {
      if (mouseY < menuStart+2*buttonHeight) {
        func = mouseX/(width/8);
      }
    }
  }
}