// Rectangular button gui element
class RectangleButton extends GuiElement {
  int textSz = 40;
  boolean _up = true;
  int _w = -1;
  int _h = -1;
  int _color = -1;
  String _txt;

  /*
   * Create rectangular button gui element
   * Parameters:
   *
   * x,y: position on screen
   * w: width of rectangular button
   * h: height of rectangular button
   * cb: callback listener
   */
  RectangleButton(int x, int y, int w, int h, int col, CallBack cb) {
    super(x, y, cb);
    _w = w;
    _h = h;
    _color = col;
    _txt = "";
  }

  // Check if mx,my is inside the gui element surface
  boolean mouseIn(int mx, int my) {
    return ((mx > _x) && (mx < _x+_w) && (my > _y) && (my < _y+_h));
  }

  // perform graphical draw actions and additionally place text
  void draw() {
    _up = getUp();
    if (_up)
      drawup();
    else
      drawdown();
    if (_txt != "") {
      fill(0);
      textSize(textSz);
      text(_txt, _x+_w/2-textWidth(_txt)/2, _y+_h/2+textSz/2);
    }
  }

  // draw gui element 'up' position
  void drawup() {
    fill(_color);
    rect(_x, _y, _w, _h);
  }

  // draw gui element 'down' position
  void drawdown() {
    fill(_color+100);
    rect(_x, _y, _w, _h);
  }

  // set text string to be displayed in the gui element
  void setText(String txt) {
    _txt = txt;
  }
}