// Slider button gui element
class Slider extends GuiElement {
  int _w = -1;
  int _h = -1;
  int _color = 100;
  float minVal = 0;
  float maxVal = 100;
  boolean horizontal = true;
  float _mx, _my;
  float val = 0;  // value set by slider
  int halfSliderWidth = 3;

  /*
   * Create slider gui element
   * Parameters:
   *
   * x,y: position on screen
   * w: width of slider
   * h: height of slider
   * cb: callback listener
   */
  Slider(int x, int y, int w, int h, CallBack cb) {
    super(x, y, cb);
    _w = w;
    _h = h;
    _mx = x;
    _my = y;
  }

  // Check if mx,my is inside the gui element surface
  boolean mouseIn(int mx, int my) {
    return ((mx > _x) && (mx < _x+_w) && (my > _y) && (my < _y+_h));
  }

  // perform draw actions
  void draw() { 
    // draw body
    fill(_color);
    rect(_x, _y, _w, _h);
    // draw slider
    fill(0);
    // put slider inside body, take slider direction and width into account
    if (horizontal) {
      _mx = max(_x+halfSliderWidth, _mx);
      _mx = min(_x+_w-halfSliderWidth, _mx);
      rect(_mx-halfSliderWidth, _y, 2*halfSliderWidth, _h);
    } else {
      _my = max(_y+halfSliderWidth, _my);
      _my = min(_y+_h-halfSliderWidth, _my);
      rect(_x, _my-halfSliderWidth, _w, 2*halfSliderWidth);
    }
  }

  void mouseDrag() {
    // call superclass method, it might perform 'mouse housekeeping' functionality
    super.mouseDrag();
    _mx = mouseX;
    _my = mouseY;
  }

  // return current slider value
  float getValue() {
    if (horizontal) return map(_mx, _x, _x+_w, minVal, maxVal);
    else return map(_my, _y, _y+_h, minVal, maxVal);
  }
}